/*
 * This class implements java.sql.Blob. 
 * The class can be freely used, distributed and modified.
 *  i-net software 2002
 */

import java.sql.*;

public class ByteArrayBlob implements java.sql.Blob
{
    byte[] bytes;
    
    public ByteArrayBlob( byte[] bytes ) throws SQLException {
        if( bytes == null )
            throw new SQLException( "[ByteArrayBlob] invalid parameter: null" );
        
        this.bytes = bytes;
    }
    
    public java.io.InputStream getBinaryStream() throws SQLException
    {
        return new java.io.ByteArrayInputStream(bytes);
    }

    public byte[] getBytes( long pos, int length ) throws SQLException
    {
        byte[] puffer = new byte[length];
        
        System.arraycopy( bytes, (int)pos - 1, puffer, 0, length);
        return puffer;
    }

    public long length() throws SQLException
    {
        return bytes.length;
    }

    public long position( Blob pattern , long start ) throws SQLException
    {
        return position( pattern.getBytes( 0, (int)pattern.length() ), start );
    }

    public long position( byte pattern[], long start ) throws SQLException
    {
    	byte v1[] = bytes;
    	byte v2[] = pattern;
    	int max = bytes.length - pattern.length;
    	
    	if( start < 0 ) {
    	    start = 0;
    	}
    	else if( start >= bytes.length ) {
    	    return -1;
    	}
    	
	    if( pattern.length == 0 ) {
	        return start;
	    }

    	int str_offset = 0;
        byte first  = v2[str_offset];
        int i = (int)start;
        
        test:
        
        while( true ) {
	        /* look for first character */
	        while( i <= max && v1[i] != first )
	            i++;
	            
	        if( i > max )
	            return -1;

	        /* found first character, now look for remainder of v2 */
	        int j = i + 1;
	        int end = j + pattern.length - 1;
	        int k = str_offset + 1;
	        
	        while( j < end ) {
		        if( v1[j++] != v2[k++] ) {
		            i++;
		            continue test;
		        }
	        }

	        return i+1;	
        }
    }
}